      subroutine ch2px4(nq,nc,ixc,iyc,ixq,iyq,ixd,iyd,ierr)
c
c     subroutine to convert EPIC chip coordinates (nq,nc,ixc,iyc)
c     into quadrant (nq,ixq,iyq) and detector (ixd,iyd) coordinates.
c                                                    K. Dennerl, Jan 97
c     call_var.        type I/O description
c 
c     nq                i4  i   quadrant number       {0,1,2,3}
c     nc                i4  i   chip     number       {0,1,2}
c     ixc               i4  i   chip x coordinate     {0..63}
c     iyc               i4  i   chip y coordinate     {0..199}
c
c     ixq               i4    o quadrant x coordinate {0..191}
c     iyq               i4    o quadrant y coordinate {0..199}
c     ixd               i4    o detector x coordinate {0..383}
c     iyd               i4    o detector y coordinate {0..399}
c     ierr              i4    o error exit
c
      implicit none
      integer*4   nq, nc, ixc, iyc, ixq, iyq, ixd, iyd, ierr
c
c     check input values:
c
      if (min(nq,nc,ixc,iyc,ixq,iyq).lt.0 .or. nq.gt.3 .or.
     &    nc.gt.2 .or. ixc.gt.63 .or. iyc.gt.199) then
        write(*,*) 'error in ch2px4: invalid input values:'
        write(*,*) nq, nc, ixc, iyc
        ierr = 2
      else
        ierr = 0
        if (mod(nq,2).eq.0) then
          ixq = 64 * (3 - nc) - ixc - 1
        else
          ixq = 64 * (nc + 1) - ixc - 1
        endif
        iyq = iyc
        if (nq.eq.0) then
          ixd = ixq
          iyd = 399 - iyq
        else if (nq.eq.1) then
          ixd = 192 + ixq
          iyd = 399 - iyq
        else if (nq.eq.2) then
          ixd = 383 - ixq
          iyd = iyq
        else
          ixd = 191 - ixq
          iyd = iyq
        endif
      endif
c
      return
      end
