C----- 2.32.1 ------------------------------------------------------------
C
C evcr03InitCteParam: initialize the CTE model parameter data needed
C                      by CTE correction routine evcr03 below; must
C                      be called prior to evcr03;
C

      subroutine evcr03InitCteParam(n, iflparin, ain, bin, cin,
     &                              tmpin, modin, y_timingin)

      implicit none
      integer n, i, j, l, k, jj, ii, icol,jm1t5
      integer iflparin(n), modin
      real    ain(n), bin(n), cin(n), tmpin, y_timingin
      real    high, rlow, rmm, rm0, rm4, rlow0, d1,d1d1

      include 'ccd_par_descr.f'
C
c     write(*,*) 'CALPNALGO-2.32.1'
C
C check input number of columns 
C
      if (n.ne.ncol) then
         write(*,*) 'evcr03InitCteParam: vector sizes wrong: ', n, ncol
         stop
      endif

      iflpar = iflparin

C
C get array of CTE model parameters
C         
      par_cte(1:n, 1) = ain(1:n)
      par_cte(1:n, 2) = bin(1:n)
      par_cte(1:n, 3) = cin(1:n)

C     do i = 1,n
C       print*,ain(i),bin(i),cin(i)
C     enddo

      temp = tmpin
      mode = modin
      y_timing = y_timingin

C
C physical parameters of the CTE model
C 
      pi      = acos(-1.0)
      rkb     = 8.6174e-5
      sigmax  = 1.2e-15
      vth     = 1e3 * sqrt(temp/300.)
      rnc     = 3.16011e23 * (temp/300.)**1.5
      et      = 0.28
      en      = vth * rnc * sigmax * exp(-et/rkb/temp)
      ap      = 3.125e4
      bp      = 4.375e5
      cp      = 0.625e7
      tzyk    = 0.08
      tphi1   = 22.5e-6
      t       = tphi1
      tsonst  = 0.02e-6
      phi_factor = 0.7e-6 / tsonst
        
      sq    = sqrt(ap*bp*cp)
      fact2 = rkb*temp
      fact1 = sq/(pi*fact2)**1.5*1e-10
      fact3 = 4e10*pi/sq*exp(-en*tphi1)
      fact4 = sqrt(fact2)
      fact5 = fact4*5.
      scale = 1.

C
C parameters depending on observational mode
C   -> these should be moved to a CCF file
C        0: Full Window
C        1: Extended Frame
C        2: Large Window
C        3: Small Window
C        4: Timing
C        5: Burst
C
      if (mode.eq.0) then
        t_frame = 0.0733
        y_shift = 0
      else if (mode.eq.1) then
C
C should be generalized depending on F1294 and number of active quadrants
C
        t_frame = 0.1992
        y_shift = 0
      else if (mode.eq.2) then
        t_frame = 0.0476
        y_shift = 100
      else if (mode.eq.3) then
        t_frame = 0.0057
        y_shift = 136
      else if (mode.eq.4) then
        t_frame = 0.00595
        y_shift = 0
        scale   = 0.43   ! temporary value for TI mode
      else if (mode.eq.5) then
        t_frame = 0.00434456
        y_shift = 0
        scale   = 1.0    ! temporary value for BU mode
      endif

      nnp = npar

C
C initial values describing hot pixels and pixels with precursors
C
        iflpr    = 0
        iflhot   = 0
        nhot_nbr = 0
        ehot     = 0.0
        ehot1    = 0.0
        iprf     = 0
        rprf     = 0
        ipry     = -1
        prel     = -1.0
        ihot     = 0

C
C calculating parameters for integration in the CTE models
C
        high = fact5
        
        do i = 1, ncol
          rlow = high*par_cte(i, 3)
          rmm  = (high-rlow)/5.
          do jj = 1, 2
             if (jj.eq.1) then
                rm0   = rmm
                rlow0 = rlow
             else
                rm0   = fact4
                rlow0 = 0.
             endif
             rm4 = rm0/4.
             do j = 1, 5
                d1 = rlow0+rm0*(j-1)-rm4
                do k = 1, 5
                  ii = (j-1)*5+k
                  d1 = d1+rm4
                  rad(i, jj, ii) = d1*d1
                  gau(i, jj, ii) = fact1*exp(-rad(i, jj, ii)/fact2)
                enddo
             enddo
           enddo
         enddo

      end


C
C evcr03InitHotPixels: initialize the list of hot pixels used inside the
C                       CTE correction routine evcr03 below; must
C                       be called prior to evcr03;
C
C

      subroutine evcr03InitHotPixels(n, iqin, icin, ichin, iyin, elin,
     &                               nhin)

      implicit none      
      integer n, i, icol
      integer iqin(n), icin(n), ichin(n), iyin(n), nhin(n)
      real    elin(n)

      include 'ccd_par_descr.f'

      nhotpixels = n
C     print*,'NHOTPIXELS =',nhotpixels

      do i = 1, n
         if (iqin(i).eq.0) then
            icol = (2-icin(i))*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.1) then
            icol = ncol_q+icin(i)*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.2) then
            icol = 2*ncol_q+(2-icin(i))*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.3) then
            icol = 3*ncol_q+icin(i)*ncol_c+ncol_c-ichin(i)
         endif
         iflhot(icol) = iyin(i)+1
         nhot_nbr(icol) = nhin(i)
         ehot(icol) = elin(i)
      enddo
C     print*,'NHOTPIXELS = DONE'

      end


C
C  evcr03InitDiscardedPixels:
C          initialize the list of discarded pixels used inside the
C          CTE correction routine evcr03 below; must
C          be called prior to evcr03;

      subroutine evcr03InitDiscardedPixels(n, iqin, icin, ichin, iyin)

      implicit none      
      integer n, i, icol
      integer iqin(n), icin(n), ichin(n), iyin(n)

      include 'ccd_par_descr.f'

      ndiscardedpixels = n
C     print*,'NDISCARDEDPIXELS =',ndiscardedpixels
      
      do i = 1, n
         if (iqin(i).eq.0) then
            icol = (2-icin(i))*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.1) then
            icol = ncol_q+icin(i)*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.2) then
            icol = 2*ncol_q+(2-icin(i))*ncol_c+ncol_c-ichin(i)
         elseif (iqin(i).eq.3) then
            icol = 3*ncol_q+icin(i)*ncol_c+ncol_c-ichin(i)
         endif
C        print*,'I,IYIN = ',i,iyin(i)
         if (iyin(i).lt.0) then
            ihot(icol, 1:nrow) = -1
         else
            ihot(icol,iyin(i)+1) = -1
         endif
      enddo
C     print*,'NDISCARDEDPIXELS = DONE'

      end



C
      subroutine evcr03(
C input
     &                  ifrm,iquad,ixq,iyq,erawp,
C output
     &                  ecor,ierr)
C
C     CTE CORRECTION OF ELECTRONS (INCLUDING PRECURSORS)
C
C     information on the event to be corrected:
C
C     ifrm   i4  input   frame number                      [1 .. infinity]
C     iquad  i4  input   quadrant id                       [0,1,2,3]
C     ixq    i4  input   quadrant column number            [0 .. 191]
C     iyq    i4  input   quadrant row number               [0 .. 199]
C     erawp  r4  input   number of electrons arriving
C                        at the CAMEX                      [0.0 .. 6000.0]
C
C     results:
C
C     ecor   r4  output  reconstructed number of electrons
C                        released by the photon
C     ierr   i4  output  error exit: 
C                                    0 - successful exit
C                                    1 - wrong pixel coordinates
C                                    2 - bright pixel/to be discarded
C                                    3 - the column has no calibration data
C                                    4 - non-positive input energy
C

      implicit none

      include 'ccd_par_descr.f'

      integer   icol, iquad, ixq, ifrm, iyq, iyq1, j, jj, nh, ny, i, npr
      integer   nh1, ji, iy1, iy2, ny11, ierr, iyq0
      real      eFF1, eFF2, eFF3
      real      tf, ty, el_old1, el_loss, ell, ecor, erawp
      real      c, d, ell0, ampl, dt, el_old2, el_old2pr, elll
      real      factor, keV_per_e, modefactor
      logical   mode1,mode2,mode3,mode23,mode4,mode5
C
C these quantities should be moved to a CCF file
C
      parameter (eFF1 = 0.55/200.0, eFF2 = eFF1/200.0*1.1)
      parameter (eFF3 = 0.05/6.0, keV_per_e = 3.72e-3)

      if (erawp .le. 0) goto 204

      ty = 0

      mode1  = (mode.eq.1)
      mode2  = (mode.eq.2)
      mode3  = (mode.eq.3)
      mode23 = (mode2.or.mode3)
      mode4  = (mode.eq.4)
      mode5  = (mode.eq.5)

C
C these quantities should be moved to a CCF file
C
      if (mode2) then
        modefactor = 0.7
      elseif (mode3) then
        modefactor = 1.0
      endif
C
C get number of the column icol
C
      icol = iquad*ncol_q+ixq+1

C
C get y-coordinate of event
C
C for TIMING and BURST mode
C
        if (mode4) then
C
C do gain-adjustment for TI mode
C
           ampl = erawp*keV_per_e
C          call SAS_GainTimingCorrect(ampl)
C          erawp = ampl/keV_per_e
           tf = ifrm*t_frame
           ty = tf+((iyq-200)-int(y_timing/10.))*29.56e-6
           iyq1 = y_timing+1.
           iyq0 = iyq+1
         elseif (mode5) then
C
C do gain-adjustment for TI mode
C
           ampl = erawp*keV_per_e
C          call SAS_GainBURSTCorrect(ampl)         
C          erawp = ampl/keV_per_e
           iyq1 = iyq+1
           iyq0 = iyq1
         else
C
C for other modes
C
           iyq1 = iyq+1
           iyq0 = iyq1
        endif

C
C get a shift in y-direction 
C
        iyq1 = iyq1 - y_shift

C
C check whether the pixel is to be discarded as hot one
C
c       print*,'IHOT: ICOL,IYQ1 = ',ICOL,IYQ1
c       if (ihot(icol,iyq1).eq.-1) go to 201
        if (ihot(icol,iyq0).eq.-1) go to 201

C
C check presence of the calibration data for the column icol,
C if no data then go to an error output
C
        if (iflpar(icol).eq.0) go to 202

C
C get model parameters for the column icol
C
        par = par_cte(icol, 1:npar)
        

C
C check presence of precursor
C
        if (iflpr(icol).gt.0) go to 101

C
C CORRECTIONS FOR EVENTS WITH NO PRECURSORS
C

C
C check presence of hot pixels
C nh is number of hot pixels
C
        nh = 0
        el_loss = 0.
        if ((iflhot(icol).eq.0).or.
     &      (iyq1.lt.iflhot(icol))) go to 401
        jj = min (iyq1, iflhot(icol)-1+nhot_nbr(icol))
        nh = 1
        do j = iflhot(icol), jj
           if (j.eq.iyq1) go to 402
           nh = nh+1
        enddo
        nh = nh-1

C
C get correction due to the hot pixels
C
402     el_old1 = ehot(icol)
        call cte_model_prec(erawp, el_old1, par, nnp, ell)
        el_loss = el_loss+ell*nh
        ehot1(icol) = ehot(icol)+ell

C
C get corrections due to the regular pixels
C
401     ell = 0.
        if (iyq1.gt.nh) then
           call cte_model_noprec(erawp, par, nnp, ell)
           el_loss = el_loss+ell*(iyq1-nh)
        endif
        
C
C store captured charge (ell) for further corrections
C
        prel(icol, iyq1) = ell
 
C
C ecor: final for FF and TI and BU mode, other modes are tuned below
C
        ecor = erawp+el_loss

C
C additional corrections for various modes
C
C for Small and Large Window modes
C
        if (mode23) then
           ampl = ecor*keV_per_e
           if (ell0.eq.0.) 
     &        call cte_model_noprec(erawp, par, nnp, ell0)
           c = factor(ampl,modefactor)
           ecor = ecor + ell0 * y_shift * c
C
C for Extended Full Frame mode
C
        elseif (mode1) then
           ampl = ecor*keV_per_e
c          c = 0.7
c          if (ampl.gt.1.0) then
c             if (ampl.le.7.0) then
c                c = 0.7-eFF3*(ampl-1.0)
c             else
c                c = 0.65
c             endif
c          endif
c c=0.6 [sas-5.1.0]
C
C the quantity C should be moved to a CCF file
C
           c = 0.56
           ecor = ecor-el_loss*(c+eFF2*iyq1*(iyq1-200.)-eFF1*iyq1)
        endif

C
C store information about this event for further corrections
C
        iflpr(icol) = 1
        ipry(icol, iyq1) = iyq1
        iprf(icol, iyq1) = ifrm
        rprf(icol, iyq1) = ty
C
C output
C
        return

C
C CORRECTIONS FOR EVENTS WITH PRECURSORS
C

C
C get number of precursors
C                 
101     npr = 0
        do i = 1, nrow
           if (ipry(icol, i).eq.-1) go to 102
           npr = npr+1
           ifp(npr) = iprf(icol, i)
           rfp(npr) = rprf(icol, i)
           iyp(npr) = ipry(icol, i)
           pre(npr) = prel(icol, i)
           if ( (mode.ne.4) .and. (ifrm.gt.iprf(icol, i)) ) go to 103
           if (i.ge.iyq1) go to 103
102     enddo

103     el_loss = 0.
        ell0 = 0.
        if (npr.gt.0) ipry(icol, iyp(1):nrow) = -1
        iyp(npr) = min (iyp(npr), iyq1)

C
C are there pixels between the event and CAMEX with no precursors?
C
410     ny = iyq1-iyp(npr)
        if (iyp(npr).eq.iyq1) go to 104

C
C electron loss in the pixels without precursors
C

C
C correction due to hot pixels
C
        nh1 = 0
        if ((iflhot(icol).eq.0).or.
     &      (iyq1.lt.iflhot(icol)).or.
     &      (iyp(npr).gt.(iflhot(icol)-1+nhot_nbr(icol)))) 
     &    go to 411
        ji = max (iyp(npr), iflhot(icol))
        jj = min (iyq1, iflhot(icol)-1+nhot_nbr(icol))
        nh1 = 1
        do j = ji, jj
           if (j.eq.iyq1) go to 412
           nh1 = nh1+1
        enddo
        nh1 = nh1-1

412     el_old1 = ehot(icol)
        call cte_model_prec(erawp, el_old1, par, nnp, ell)
        el_loss = el_loss+ell*nh1
        ehot1(icol) = ehot(icol)+ell

411     ell = 0.
        if (ny.gt.nh1) then
           call cte_model_noprec(erawp, par, nnp, ell)
           el_loss = el_loss+ell*(ny-nh1)
        endif
        ell0 = ell
        prel(icol, iyq1) = ell
        iprf(icol, iyq1) = ifrm
        ipry(icol, iyq1) = iyq1
        rprf(icol, iyq1) = ty

C
C account for precursors 
C
104     el_old2pr = 0.
        do i = npr, 1, -1
           if (mode4) then
              dt = ty-rfp(i)
              el_old2 = exp(-en*dt)*pre(i)
           else
              if (ifrm.gt.ifp(i)) then
                 el_old2 = 0.
              else
                 el_old2 = pre(i)
              endif
           endif
           nh = 0
           iy1 = 0
           if (i.gt.1) iy1=iyp(i-1)
           iy2 = iyp(i)
           if ((iflhot(icol).eq.0).or.
     &         (iy2.lt.iflhot(icol)).or.
     &         (iy1.gt.(iflhot(icol)-1+nhot_nbr(icol))))
     &     go to 421
           ji = max (iy1, iflhot(icol))
           jj = min (iy2, iflhot(icol)-1+nhot_nbr(icol))
           nh = jj-ji+1
           el_old1 = el_old2
           if (el_old2.lt.ehot(icol)) el_old1=ehot(icol)
           call cte_model_prec(erawp, el_old1, par, nnp, ell)
           el_loss = el_loss+ell*nh
        
421        if (el_old2.eq.0.) then
              if (ell0.eq.0.) 
     &        call cte_model_noprec(erawp, par, nnp, ell0)
              ell = ell0
           else
              if (i.eq.npr) then
                 call 
     &           cte_model_prec(erawp, el_old2, par, nnp, elll)
                 el_old2pr = el_old2
              else
                 if (el_old2.ne.el_old2pr) then
                    call 
     &              cte_model_prec(erawp, el_old2, par, nnp, elll)
                    el_old2pr = el_old2
                 endif
              endif
              ell = elll
           endif

           if (i.eq.1) then
              ny11 = iyp(1)
           else
              ny11 = iyp(i)-iyp(i-1)
           endif
           if (ny11.le.nh) go to 415
           el_loss = el_loss+ell*(ny11-nh)
415        iprf(icol, iyp(i)) = ifrm
           ipry(icol, iyp(i)) = iyp(i)
           prel(icol, iyp(i))= ell
           rprf(icol, iyp(i)) = ty
        enddo

        ecor = erawp+el_loss
C
C additional corrections for various modes
C
C for Small and Large Window modes
C
        if (mode23) then
           ampl = ecor*keV_per_e
           if (ell0.eq.0.)
     &        call cte_model_noprec(erawp, par, nnp, ell0)
           c = factor(ampl,modefactor)
           ecor = ecor + ell0 * y_shift * c
C
C for Extended Full Frame mode
C
        elseif (mode1) then
           ampl = ecor*keV_per_e
c          c = 0.7
c          if (ampl.gt.1.0) then
c             if (ampl.le.7.0) then
c                c = 0.7-eFF3*(ampl-1.0)
c             else
c                c = 0.65
c             endif
c          endif
c c=0.6 [sas-5.1.0]
           c = 0.56
           ecor = ecor-el_loss*(c+eFF2*iyq1*(iyq1-200.)-eFF1*iyq1)
        endif

C
C no-error return
C
200     ierr=0
        return

C
C returns with errors
C
C discarded pixel
201     ierr=2
        return
C no calibration data for this column
202     ierr=3
        return
C negative input energy
204     ierr=4
        return

        end





      function factor(e,modefactor)
C
C function for correction in SW and LW mode
C
      implicit none
      real factor, e, modefactor,a0,a11,a12,a13
C
C these numbers should be moved to a CCF
C
      parameter (a0 = 1.0, a11 = -0.225, a12 = +1.3, a13 = 5.5)

c     factor = 0.0
c     if (e.ge.0.1) then
c
c           factor = 1.016
c    &              -13.16/(1.+((e+0.2606)/0.07505   )**2)
c    &          -9.4787e-2/(1.+((e-1.427) /2.202     )**2)
c    &               -1.59/(1.+((e+3.489) /2.04      )**2)
c    &          +4.6923e-2/(1.+((e-7.815) /2.477     )**2)
c
c           factor = factor * modefactor
c
c     endif
c     factor = (1.0-0.2*exp(-((e-1.8)/0.7)**2))*modefactor
c     factor = (1.0-0.225*exp(-((e-1.3)/5.5)**2))*modefactor
      factor = (a0+a11*exp(-((e-a12)/a13)**2))*modefactor

      end
