C-UL
C-UL    evgn01init: This routine replaces the reading of the gain
C-UL                calibration data in the original code
C-UL                (see comments C-UL below); it must be called
C-UL                prior to calling `evgn01' to place needed data in
C-UL                the common block `GAINPARAM'
C-UL
C-UL    CHANGE HISTORY:
C-UL        UL, 1999-06-15: 1.0.0: - first version
C-UL
C-UL    AUTHOR:
C-UL        UL, Wed Jun 16 11:18:28 MET DST 1999
C-UL
C-KD        parameters m, gnin added for final gain adjustment
C-KD
      subroutine evgn01init(n, pp0in, pp1in, pp2in, pp3in, aain, bbin,
     &                      m, gnin)
      implicit none

      integer*4 n, i, m
      real*4 pp0in(n), pp1in(n), pp2in(n), pp3in(n), aain(n), bbin(n),
     &       gnin(m)

      include 'gainparam.f'

      !
      !  check that vector sizes are ok
      !
      if (n.ne.ncol) then
          write(6, *) 'evgn01init: vector sizes do not match: ', n, ncol
          stop
      endif

      do i = 1, n
         pp0(i) = pp0in(i)
         pp1(i) = pp1in(i)
         pp2(i) = pp2in(i)
         pp3(i) = pp3in(i)
         aa(i)  = aain(i)
         bb(i)  = bbin(i)
      enddo

      do i = 1, m
        gn(i) = gnin(i)
      enddo

      end
c
c
c      
cc      subroutine evgn01(iquad,iccd,ixc,iamt,iamp,ampc,ierr) ! old par list
cc    subroutine evgn01(iquad,iccd,ixc,iyc,iamp,ampc,ampcc,ierr) ! old par list
      subroutine evgn01(iquad,iccd,ixc,iyc,ampr,ampc,ampcc,ierr)
c
c-----------------------------------------------------------------------
c     gain correction with approximate CTE correction
c
c     For each readout channel the linearized gain is calculated
c     which yields  1 adu = 5.00 eV  for E > 2 keV.
c     The gain was found to depend on the charge released by the
c     photon. Therefore, in addition to the charge of the event
c     to be corrected (iamp), the subroutine requires also the total
c     charge released by the photon (iamt), to estimate its energy.
c     In addition to the gain correction an approximate CTE correction
c     will be performed. This correction takes the energy dependence
c     of the CTE into account, but not the effect of precursors
c
c     chgn   c   input   gain calibration data file
c     iquad  i4  input   quadrant id                    (0,1,2,3)
c     iccd   i4  input   ccd id                          (0,1,2)
c     ixc    i4  input   ccd column number               (0..63)
c     iyc    i4  input   ccd row number                  (0..199)
c     apmr   r4  input   raw amplitude to be corrected    [adu]
c     ampc   r4  output  gain corrected amplitude         [adu]
c     ampcc  r4  output  gain and CTE corrected amplitude [adu]
c     ierr   i4  output  error exit            (ierr=0: no error)
c
c                                                     KD_MPE_99_05
c                                                     KD_MPE_00_03
c-----------------------------------------------------------------------
c
      implicit none
      real*4        ampc, ampcc, e, h, p0, p1, p2, p3,
     &              x, y, ampr
      integer*4     iquad, iccd, ixc, iyc, ierr, k1, k2,
     &              icol, ixq, iyq            !   , iamp
c
      include 'gainparam.f'
c
      save
c
      ierr = 0
c
c-----------------------------------------------------------------------
c
c     convert the ccd coordinates into 'column coordinates':
c
      call ch2px4(iquad,iccd,ixc,0,ixq,iyq,k1,k2,ierr)
c
      icol = iquad * nxdim + ixq + 1
c
c-----------------------------------------------------------------------
c
c     correct the amplitude for gain:
c
      if (ierr.ne.0) then
c
        ampc  = 0.0
        ampcc = 0.0
c
      else
c
c       ampr = real(iamp)                       ! obsolete @ 31-Jul-2000 by mjf
c       if (ampr .lt. 0) then
c         write(*,*) 'EVGN01: E < 0: iquad,iccd,ixc,iyc,ampr = ',
c    &                               iquad,iccd,ixc,iyc,ampr
c         ierr = 3                   ! changed, 31-Aug-2001
c         return
c       endif
          
        h = ampr * aa(icol)    ! rough gain correction         ! adapted by mjf
        e = h * 5.e-3                ! approximate keV value
c
        if (e.le.0.0) then
          ampc = ampr                                          ! adapted by mjf
          ampcc = ampc               ! changed, 12-Apr-2000
          ierr = 3                   ! changed, 12-Apr-2000
c         write(*,*) 'EVGN01: AA < 0: icol,aa(icol),ampr,e = ',
c    &                                icol,aa(icol),ampr,e
          return
        endif            
c    
        x = alog10(e)
c
        p0 = pp0(icol)
        p1 = pp1(icol)
        p2 = pp2(icol)
        p3 = pp3(icol)
c
        y =  ( ( p3 * x + p2 ) * x + p1 ) * x + p0
c
        ampc = aa(icol) * ampr * y ** bb(icol)                 ! adapted by mjf
c
C-KD    the following line was corrected on 3-Dec-1999:
c
        ampc = ampc * (1.0 + gn(1) * e**gn(2)) - gn(3) * e
c
c-KD    the following line was added on 14-Mar-2000:
c
        ampcc = ampc / y ** real(iyc)
c
      endif
c
c-----------------------------------------------------------------------
c
      return
      end
