module sas_epn_gaincticorrect_mod

  use dal
  use errorHandling

  implicit none

  public

contains

! ===================================
! additional gain and CTI corrections
! ===================================
subroutine SAS_pnGainCtiCorrect(ampl,rawy,ccd,temperature_gain,tstart,tstop, &
                              do_tempcorr,do_ccdoffs,do_lt_cti_soft, &
                              do_lt_cti_y, do_lt_cti, corrpar, nevent, &
			      corrpar_in)

  real(kind=single),   dimension(:), pointer   :: ampl
  integer(kind=Int16), dimension(:), pointer   :: rawY ! [1..200]
  integer          , intent(in)     :: ccd, nevent
  logical          , intent(in)     :: do_tempcorr, do_ccdoffs, &
                                       do_lt_cti_soft, do_lt_cti_y, do_lt_cti
  real(kind=single), intent(in)     :: temperature_gain
  real(kind=double), intent(in)     :: tstart, tstop, corrpar_in(5)
  real(kind=double), intent(out)    :: corrpar(12)
  real(kind=double)                 :: t, tau, aduref, tau0, tau1(0:2)
  real(kind=double)                 :: tt = 1.d0, logkeV = 0.0d0, lty0
  integer                           :: i = 0
  real(kind=single)                 :: aa(1:12), bb(1:12), cc(1:12), og(1:12)
!
  data aa / -3.800354, -4.765878, -3.111817, &
            -4.776627, -5.182015, -3.678885, &
            -2.681449, -2.714766, -2.240395, &
            -4.474558, -4.031777, -3.849648 /
!-------
! old values for quadrant 0 temperatures only
!   data bb / -3.5907, -1.9972, -3.5234, &
!             -3.3272, -4.4088, -7.0909, &
!             -2.3641, -4.6212, -4.5138, &
!              0.9298, -0.0500, -1.2397 /
!
!   data cc / 0.4774, 0.4448, 0.4170, &
!             0.5201, 0.5856, 0.6368, &
!             0.3482, 0.4528, 0.4326, &
!             0.2544, 0.3164, 0.3454 /
!-------
  data bb / -3.4429, -1.7452, -3.0335, &
            -3.1745, -3.8628, -6.5703, &
            -1.9575, -3.8182, -3.8192, &
            +1.6026, +0.8193, -0.2265 /
!
  data cc / 0.4669, 0.4210, 0.3724, &
            0.4573, 0.4816, 0.5220, &
            0.3033, 0.3677, 0.3535, &
            0.1954, 0.2305, 0.2431 /
!-------
  data og / 0.9665, 0.9661, 0.9640, &
            0.9672, 0.9663, 0.9637, &
            0.9695, 0.9691, 0.9684, &
            0.9080, 0.9102, 0.9104 /
!
! data lty0   / 120.0d0 /   ! reference Y position for long-term correction
! data tau    / 2.56d0 /    ! shift channels per year of reference channel
! data tau0   / 2.54d0 /    ! shift channels per year of reference channel
! data tau1   / 1.15d0, -0.26d0, 0.10d0 /    ! energy dependent factor
  data aduref / 1179.6d0 /  ! reference channel [Mn K-alpha]

  call message(AppMsg,Noisy,"... entering SAS_GainCtiCorrect ..")

  t = (0.5d0*(TSTART+TSTOP)/3.1536D+07)-2.0d0   ! years since 1.1.2000

  corrpar(1) = og(ccd)
  corrpar(2) = t
  corrpar(3) = aduref
  corrpar(8) = temperature_gain
!
! energy rescaling compared to Orsay calibration
!
  do i = 0,nevent-1
    ampl(i) = ampl(i) * og(ccd)
  enddo
!
! correct for long-term CTE decrease (modeled as gain):
!
  logkeV = 0.0

  tau     = corrpar_in(1)
  tau0    = corrpar_in(1)
  tau1(0) = corrpar_in(2)
  tau1(1) = corrpar_in(3)
  tau1(2) = corrpar_in(4)
  lty0    = corrpar_in(5)

  if (do_lt_cti) then
    if (do_lt_cti_soft) then
      do i = 0,nevent-1
        logkeV = log10(ampl(i)*5.0d-3)
        tt = tau1(0)+logkeV*(tau1(1)+tau1(2)*logkeV)
        if (do_lt_cti_y) tt = tt*rawy(i)/lty0
        ampl(i) = ampl(i) * (1.0d0 + t * tau0 * tt/aduref)
      enddo
      corrpar(4)  = tau0
      corrpar(9)  = tau1(0)
      corrpar(10) = tau1(1)
      corrpar(11) = tau1(2)
    else
      do i = 0,nevent-1
        tt = 1.0d0
        if (do_lt_cti_y) tt = tt*rawy(i)/lty0
        ampl(i) = ampl(i) * (1.0d0 + t * tau  * tt/aduref)
      enddo
      corrpar(4)  = tau
      corrpar(9)  = 0
      corrpar(10) = 0
      corrpar(11) = 0
    endif
    if (do_lt_cti_y) then
      corrpar(12) = lty0
    else
      corrpar(12) = 0
    endif
  else
    corrpar(4)  = 0
    corrpar(9)  = 0
    corrpar(10) = 0
    corrpar(11) = 0
    corrpar(12) = 0
  endif

  if (do_ccdoffs) then
!
! correct for CCD offset (modeled as gain):
!
    do i = 0,nevent-1
      ampl(i) = ampl(i) - aa(ccd)
    enddo
    corrpar(5) = aa(ccd)
  else
    corrpar(5) = 0.0
  endif
 
  if (do_tempcorr) then
!
! correct for temperature variations (modeled as gain):
!
    do i = 0,nevent-1
      ampl(i) = ampl(i) * (1.0d0 - (bb(ccd) + temperature_gain * cc(ccd))/aduref)
    enddo
    corrpar(6) = bb(ccd)
    corrpar(7) = cc(ccd)
  else
    corrpar(6) = 0.0
    corrpar(7) = 0.0
  endif

  if (verbosity(AppMsg) >= Verbose) then
    call message(AppMsg,Sparse,' ')
    write(errstr,'(3x,a,i16.1)') 'number of CCD to be corrected =',ccd
    call message(AppMsg,Sparse)
    write(errstr,'(3x,a,f16.4)') 'Start time                    =',TSTART
    call message(AppMsg,Sparse)
    write(errstr,'(3x,a,f16.4)') 'Stop time                     =',TSTOP
    call message(AppMsg,Sparse)
    write(errstr,'(3x,a,f16.4)') 'Orsay gain scaling            =',og(ccd)
    call message(AppMsg,Sparse)
    write(errstr,'(3x,a,f16.8)') 'Years elapsed since 1.1.2000  =',t
    call message(AppMsg,Sparse)
    if (do_lt_cti) then
      if (do_lt_cti_soft) then
        write(errstr,'(3x,a,f16.4)') 'CTI increase [adu/yr]         =',tau0
        call message(AppMsg,Sparse)
        write(errstr,'(3x,a,f16.3)') ' energy dependence: coeff(0)  =',tau1(0)
        call message(AppMsg,Sparse)
        write(errstr,'(3x,a,f16.3)') ' energy dependence: coeff(1)  =',tau1(1)
        call message(AppMsg,Sparse)
        write(errstr,'(3x,a,f16.3)') ' energy dependence: coeff(2)  =',tau1(2)
        call message(AppMsg,Sparse)
      else
        write(errstr,'(3x,a,f16.4)') 'CTI increase [adu/yr]         =',tau
        call message(AppMsg,Sparse)
      endif
    endif
    if (do_ccdoffs) then
      write(errstr,'(3x,a,f16.6)') 'CCD offset [adu]              =',aa(ccd)
      call message(AppMsg,Sparse)
    endif
    if (do_tempcorr) then
      write(errstr,'(3x,a,f16.3)') 'Temperature for gain [deg C]  =',temperature_gain
      call message(AppMsg,Sparse)
      write(errstr,'(3x,a,f16.4)') 'Temperature offset [adu]      =',bb(ccd)
      call message(AppMsg,Sparse)
      write(errstr,'(3x,a,f16.4)') 'Temperature slope [adu/K]     =',cc(ccd)
      call message(AppMsg,Sparse)
    endif
    call message(AppMsg,Sparse,' ')
  endif

end subroutine SAS_pnGainCtiCorrect



! ==========================================
! additional gain adjustment for TIMING mode
! ==========================================
subroutine SAS_pnGainTimingCorrect(ampl,nevent)

  real(kind=single), dimension(:),pointer  :: ampl     ! in [adu] = [5eV]
  integer                                  :: ierr = 0, i = 0, nevent
  real(kind=double)                        :: a0 = 0.0, h = 0.0, e_eV = 0.0
! real(kind=double)                        :: ekev = 0.0
! real(kind=double),parameter              :: a1 = 1.012d0, a2 = 0.99091d0
! real(kind=double),parameter              :: x0 = 1281.14343d0, dx = 308.3543d0
  real(kind=double),parameter              :: a1 = 1.01594d0, a2 = 0.99134d0
  real(kind=double),parameter              :: x0 = 1031.56377d0, dx = 401.47087d0
!
! fine-adjustment of the gain in TIMING mode to be applied after all
!   the gain corrections or immediately before the CTI corrections
! the input to the routine is in [adu], the coefficients have been derived
!   for energies in [keV], therefore scale input by 1 adu = 5.e-3 keV
!
  a0 = a1-a2
  h = 0.0
  do i = 0,nevent-1
    e_eV = ampl(i)*5.d-0
    h = a2 + a0/(1.d0+exp((e_eV-x0)/dx))      ! kirsch fs
    ampl(i) = ampl(i)/h                       ! ti/ff
    ierr = 0
  enddo

end subroutine SAS_pnGainTimingCorrect



! ==========================================
! additional gain adjustment for BURST mode
! ==========================================
subroutine SAS_pnGainBurstCorrect(ampl,nevent)

  real(kind=single), dimension(:),pointer  :: ampl     ! in [adu] = [5eV]
  integer                                  :: ierr = 0, i = 0, nevent
  real(kind=double)                        :: a0 = 0.0, h = 0.0, e_eV = 0.0
! real(kind=double),parameter              :: a1 = 1.04144d0, a2 = 1.00753d0
  real(kind=double),parameter              :: a1 = 1.039357d0, a2 = 1.00753d0
! real(kind=double),parameter              :: x0 = 1006.08475d0, dx = 661.2341d0
  real(kind=double),parameter              :: x0 = 4728.598d0, dx = 661.2341d0
!
! fine-adjustment of the gain in BURST mode to be applied after all
!   the gain corrections or immediately before the CTI corrections
! the input to the routine is in [adu], the coefficients have been derived
!   for energies in [eV], therefore scale input by 1 adu = 5.d-0 eV
!
  a0 = a1-a2
  do i = 0,nevent-1
    e_eV = ampl(i)*5.d-0
    h = a2 + a0/(1.+exp((e_eV-x0)/dx))        ! kirsch fs
    ampl(i) = ampl(i)*h                       ! sw/bu
    ierr = 0
  enddo

end subroutine SAS_pnGainBurstCorrect


end module sas_epn_gaincticorrect_mod
