      subroutine getpnrsp( 
     & energy_lo, energy_hi, 
     & nchannel,
     & emin, emax,
     & iline,
     & cmode,
     & pattern,
     & pattfracchan,
     & pattfraction,
     & nsiabs, sienergy, siabs,
     & nsio2abs, sio2energy, sio2abs,
     & response )
C-HS
C-HS   Modified by H Siddiqui from epic_pn_rsp by F Haberl
C-HS

C
C program to create the EPIC PN response matrix as FITS binary table
C
C version 1.0 fwh 14-feb-1997 template program just to create the FITS table
C                             and fill in keywords used in calibration files
C
C version 1.1 fwh 19-feb-1997 gauss approximation of response distribution
C                             in channels, FWHM scales as constant+sqrt(E)
C                             giving FWHM for Fe(6.4keV) about 165 eV
C                             and         for Al(1.5keV) about 108 eV
C
C                             to roughly fit the Fe spectrum
C                             the following contributions are included:
C                             escape with constant fraction
C                             constant level of partials 
C                             noise at low channels (exponential fall off)
C
C version 2.0 fwh 20-nov-1998 approximations are replaced by 
C                             Partial Event Model (PEM)
C
C version 2.1 fwh 26-apr-1999 Energy dependence of PEM parameters included.
C                             The values are preliminary (determined from
C                             data with preliminary CTE correction)
C
C version 2.2 fwh 31-may-1999 PEM parameters updated after better removal of 
C                             out-of-time events.
C
C version 3.0 fwh 16-feb-2000 PEM parameters updated, now based on Orsay 
C                             data of FS corrected with official CTE model
C                             (updated in subroutine pem_par.f)
C
C RDS: Version 2.0, 15-April-2001 - from version 5.0 fwh 22-Jan-2001
C                             Only includes Pattern zero code so far.
C                             Needs to have cmode as a parameter.
C UL: Version 2.0.1, 23-April-2001 - added cmode parameters
C                                    CAL will set this to
C                                     'ff': full-frame/extended full-frame mode
C                                     'sw': all other modes
C 	
C RDS: Version 2.0.2, 28-April-2001 - added extra renormalisation.
C UL: Version 2.1, 14-Jan-2001: added parameters pattern/pattfraction
C RDS: Version 2.2, 17-Jan-2001: added new code from epic_pn_rsp v6.2
C                                + final normalisation
C
C     Input parameters for which spectral response should be calculated

      implicit none

C-HS  Energy range over which the response is defined
      REAL*4    energy_lo, energy_hi

C-HS  Number of channels in the response
      INTEGER   nchannel

C-HS  Energy bounds for channels
      REAL*4    emin(nchannel), emax(nchannel)

C-RDS Limits:
C      INTEGER nenergy_s, minchan

C-HS  The CCD line over which the response is valid
      INTEGER  iline

C-UL  CCD mode: 'ff'  : full-frame mode
C-UL            'ef'  : extended full-frame mode
C-UL            'sw'  : small window
C-UL            'lw'  : large window 

      CHARACTER*2 cmode


C-UL  pattern: pattern to consider in spectrum:
C-UL    SINGLE	= 0,	//	singles
C-UL    DOUBLE	= 1,	//	doubles
C-UL    TRIPLE	= 2,	//	triples,
C-UL	QUADRUPLE = 3,	//	quadruples
C-UL    SD		 = 4,	//	singles+doubles
C-UL    SDTQ	= 5,	//	singles+doubles+triples+quadruples
C-UL
C-UL  pattfraction: vector with pattern fraction values
      INTEGER  pattern, ipat, ipats, ipate, pattfracchan
      REAL*4   pattfraction(pattfracchan, *)

C     storage arrays for Si and SiO2  absorption data
C     these are read from ASCCI files by subroutine readabs
      INTEGER*4 nsiabs, nsio2abs
      REAL*4    sienergy(nsiabs), siabs(nsiabs)
      REAL*4    sio2energy(nsiabs), sio2abs(nsio2abs)

C-RDS  Array to store the response per event grade
      real*4    partresp(4,nchannel)

C-HS  Output response vector
      real*4    response(nchannel)

C     Energy, electronic noise, length of transisition region, model parameters c,tau,s
      REAL*4      e_0_in, s_el_in, l_in, c_in, tau_in, s_in
      REAL*4      tau_in_line, e_0_med

      integer   channel(nchannel)
      real*4    rchannel(nchannel)
      real*4    ec_conversion, ecmax

      INTEGER*4 e_max
      PARAMETER (e_max=40000)
      REAL*8 e_ax(0:e_max-1)
      REAL*4 ec_ax(0:e_max-1), c_ax(0:e_max-1)
      REAL*4 ec1, ec2
      REAL*8 espectrum(0:e_max-1)
      REAL*8 esc_spectrum(0:e_max-1)
C-HS       REAL*4 cspectrum(nchannel)

      integer i, j, k, kmin, kmax, kcmax
C-HS       integer*4 lenact, iparse, nret, iflag, aflag
C-HS       character*40 outline
      real*4 e1, e2
      real*8 y, sum1, totsum
      real*4 sum
      real*4 z

C-HS       real*4 lblock
      real*4 line
      integer*4 e_bin

c     detector id:   1 for FM
c                    2 for FS (flight detector)

c      integer thresh,dthresh              ! RDS v2.0 - Minimum channel to use

c     parameters for escape - Peak
      REAL*4 i_esc, a_l, e_k, e_esc, i_fac
      DATA i_esc/0.023/           ! RDS v2.0
      DATA a_l/-3.92e-4/          ! RDS v2.0 
      DATA e_k/1844./
      DATA e_esc/1740./

      DATA e_bin/1/
c      DATA thresh/20/             
c      DATA dthresh/40/            

C Convert pattern into grades wanted
      if (pattern .lt. 4) then
        ipats = pattern+1
        ipate = ipats
       elseif (pattern.eq.4) then
        ipats = 1
        ipate = 2
       elseif (pattern.eq.5) then
        ipats = 1
C     This should be ipate=4 but no rmf is yet implemented for 
C     trips or quads so set to 2
        ipate = 2 
      endif

C-HS  Convert iline to real
      line = real(iline)

c     get model input parameters

c      version 2.0: some average values
c      s_el_in = 9.0
c      l_in = 50.
c      c_in = 1.22
c      tau_in = 100.
c      s_in = 0.45

C     initialize the channel boundaries
C     using crude E-channel conversion (keV per channel)
c      ec_conversion = 0.00531
c     from FM* ORSAY 6000eV measurement PN980605.541 Q0 CCD0
c      ec_conversion = 0.005086
c     version 2.1 and 2.2:
c     E-channel conversion fixed at 0.005 after CTE correction
c      ec_conversion = 0.005
          
c     energy axis for PEM (1eV bins)
c     for the time being E-channel conversion energy dependent
c
C     Find the energies of the PI channels on the fly. This used
C     to be coded into the CCF but is now mode dependent

      if (cmode.eq.'ff') then
         ec1 = 642.0
         ec2 = 0.500
      elseif (cmode.eq.'ef') then
         ec1 = 580.6
         ec2 = 0.190
      elseif (cmode.eq.'sw') then
         ec1 = 642.0
         ec2 = 0.500
      elseif (cmode.eq.'lw') then
         ec1 = 642.0
         ec2 = 0.500
      else
         ec1 = 642.0
         ec2 = 0.500
      endif
      z = ec1*ec1*2.
      do j=0,e_max-1
        e_ax(j) = real(j)
        ec_ax(j) = float(j)
        ec_conversion = 5.0 + ec2*exp(-ec_ax(j)*ec_ax(j)/z)

        c_ax(j) = ec_ax(j)/ec_conversion
c      write(*,*) ec_ax(j), ec_conversion
      enddo

      do j=1,nchannel
        channel(j) = j-1
        rchannel(j) = float(channel(j))
      end do
c     interpolate the channel axis
      emin(1) = 0.
      do j=2,nchannel
        call findindex(c_ax, e_max, rchannel(j), i)
        if (i .eq. e_max) then
           emin(j) = ec_ax(i)
        else
           call lininterp(c_ax(i), c_ax(i+1), ec_ax(i), ec_ax(i+1),
     &                  rchannel(j), z)
           emin(j) = z
        endif
        emax(j-1) = emin(j)
c        write(*,*) j-1, emin(j-1), emax(j-1), emax(j-1)/float(j-1)
      end do
      j = nchannel
      emax(j) = emin(j) + emax(j-1) - emin(j-1)
c      write(*,*) j, emin(j), emax(j), emax(j)/float(j)

c     convert to eV
      ecmax = emax(nchannel)
      kcmax = int(ecmax)

c debug !!!!!
c      write(*,*) (j, emin(j), emax(j), j=190,210)
c debug !!!!!

      totsum=0.0d0

c     start of pattern loop
      do ipat = ipats,ipate
      if (ipat.eq.1) then
c        minchan=thresh
c        write(*,*)'Using noise parameters for single-pixel events'
      elseif (ipat.eq.2) then
c        do i=1,nenergy
c         no contribution to response for energies below 170 eV
c          if(energy_evhi(i).lt.170.0)nenergy_s=i
c        enddo
c        minchan=dthresh
c        write(*,*)'Using noise parameters for double-pixel events'
      endif

C-HS  Generate response for energies energy_lo to energy_hi
C-HS  (Response generated half-way between these energies)

        e_0_in = (energy_lo +  energy_hi) / 2
c
c
c       calculation of PEM:
c       assumes 1eV steps in spectrum
c
c       from version 2.1 on:
c       get PEM parameters (dependent on energy and CCD line)
        call pem_par (e_0_in, ipat, line, s_el_in, l_in,
     &                c_in, tau_in, s_in, cmode)
        tau_in_line = tau_in
c       calculate model spectrum 
        call pem_model(sienergy, siabs, nsiabs, 
     &             sio2energy, sio2abs, nsio2abs,
     &             e_0_in, s_el_in, l_in, c_in, tau_in, s_in,
     &             e_ax, espectrum)
c       normalisation to integral equal unity:
c       (only up to energy covered by last channel)
        sum1 = 0.d0
        do j=0,kcmax
          if (espectrum(j).lt.1.e-9)then
            espectrum(j)=0.d0
          else
            sum1 = sum1 + espectrum(j)
          endif
        enddo
        do j=kcmax+1,e_max-1
          espectrum(j) = 0.d0
        enddo
        do j=0,kcmax
          espectrum(j) = espectrum(j)/sum1
        enddo
c        write(*,'(f10.2,f20.10)') (e_ax(j), espectrum(j), j=0,kcmax)

c------------------------------------------------------------------------------+
c       Add Escape - Peak, if necessary:
c------------------------------------------------------------------------------+
        if (e_0_in.gt.e_k+e_bin) then
          i_fac = i_esc*exp(a_l*(e_0_in-e_k))
          e_0_in = e_0_in-e_esc
          e_0_med = e_0_in + e_esc*0.5
c         get PEM parameters for escape line
c         version 3.0: use medium energy for PEM parameters
c                      and tau at line energy
          call pem_par (e_0_med, ipat, line, s_el_in, l_in, c_in, 
     &                  tau_in, s_in, cmode)
          tau_in = tau_in_line
c         calculate escape model spectrum 
          call pem_model(sienergy, siabs, nsiabs, 
     &             sio2energy, sio2abs, nsio2abs,
     &             e_0_in, s_el_in, l_in, c_in, tau_in, s_in,
     &             e_ax, esc_spectrum)
c         normalisation to integral equal unity:
c         (only up to energy covered by last channel)
          sum1 = 0.d0
          do j=0,kcmax
            if (esc_spectrum(j).lt.1.e-9)then
              esc_spectrum(j)=0.d0
            else
              sum1 = sum1 + esc_spectrum(j)
            endif
          enddo
          do j=kcmax+1,e_max-1
            esc_spectrum(j) = 0.d0
          enddo
          do j=0,kcmax
            esc_spectrum(j) = esc_spectrum(j)/sum1
          enddo
c          write(*,'(f10.2,f20.10)') (e_ax(j),esc_spectrum(j),j=0,kcmax)

c         add escape peak
          do j=0,kcmax
            espectrum(j) = (1.-i_fac)*espectrum(j) + 
     &                   i_fac*esc_spectrum(j)
          enddo
c
c         normalize whole spectrum:
          sum1 = 0.d0
          do j=0,kcmax
c            if (espectrum(j).lt.1.e-9)then
            if (espectrum(j).lt.1.e-8)then
              espectrum(j)=0.d0
            else
              sum1 = sum1 + espectrum(j)
            endif
          enddo
          do j=kcmax+1,e_max-1
            espectrum(j) = 0.d0
          enddo
          do j=0,kcmax
            espectrum(j) = espectrum(j)/sum1
          enddo
c          write(*,'(f10.2,f20.10)') (e_ax(j), espectrum(j), j=0,kcmax)

        endif

        sum = 0.0

c       rebin and normalise PEM spectrum, using minchan - RDS v2.0
c        do j=1,minchan-1
c          response(j) = 0.0
c        enddo
        do j=1,nchannel                                                   
           e1 = emin(j)
           e2 = emax(j)
           kmin = int(e1)
           kmax = int(e2)
           y = espectrum(kmin)*(e_ax(kmin+1)-e1)
           do k=kmin+1,kmax-1
              y = y + espectrum(k)
           enddo
           y = y + espectrum(kmax)*(e2-e_ax(kmax))
           partresp(ipat,j) = real(y)
           sum = sum + partresp(ipat,j)
        end do
        do j=1,nchannel
          partresp(ipat,j) = partresp(ipat,j)/sum
        end do

c     pattern fraction correction in channel space:
C        call patmod(patfraction, ccdmode, dthresh, ipat, line)

        do j = 1,nchannel
          partresp(ipat,j) = partresp(ipat,j)*pattfraction(j,ipat)
          totsum = totsum + partresp(ipat,j)
        end do

      enddo
c    end pattern loop

c    sum up pattern rmfs and normalise output RMF.
      do j=1,nchannel
         response(j)=0.0
      enddo
 
      do j=1,nchannel
         do ipat=ipats,ipate
            response(j) = response(j) + partresp(ipat,j) 
         enddo
         response(j) = response(j) / totsum
      enddo

      end
