C-HS =========================================================
C-HS
C-HS     pem_model.f
C-HS
C-HS Slightly modified version of F Haberl's pem_e.f
C-HS
C-HS This version does not use the variable nene,
C-HS as the variables nsiabs and nsio2abs already
C-HS describe the number of data points in each absorption curve.
C-HS
C-HS Numerical Recipe calls are replaced by hand-written or SLATEC code.
C-HS
C-HS First version by H Siddiqui, ESTEC 27 June 2000
C-RDS Version 2: 2 April 2001 - update from Frank Haberl
C-HS =========================================================

      subroutine pem_model(sienergy, siabs, nsiabs, 
     &     sio2energy, sio2abs, nsio2abs,
     &     e_0_in, s_el_in, l_in, c_in, tau_in, s_in,
     &     e_ax, spec_noise)


c     Calculate pn - EPIC Energy response functions analytically
c     following an extended partial event model
c     Converted from IDL program of Martin Popp
c     Frank Haberl Dec. 1998

c     Input parameters for which spectral response should be calculated
c     Energy, electronic noise, length of transisition region, model parameters c,tau,s
      REAL*4      e_0_in, s_el_in, l_in, c_in, tau_in, s_in
      REAL*8      e_0,    s_el,    l,    c,    tau,    s

      REAL*8  a_cce, b_cce

c     storage arrays for Si and SiO2  absorption data
      INTEGER*4   nsiabs, nsio2abs
      REAL*4      sienergy(nsiabs), siabs(nsiabs)
      REAL*4      sio2energy(nsio2abs), sio2abs(nsio2abs)

c     interpolated absorption lengths for given energy
      REAL*8      a_si, a_sio2

C-HS  For iterpolating between points
      REAL*4     yvalue

c     local settings

      REAL*4 d_sio2, d_si, w, f_fano, s_cte

      REAL*8 n_ev, n_frames, num_pixels, n_noise
      REAL*4 n_sio2_dead, n_sio2_shelf, n_si_dead
      REAL*4 n_rest, n_si, n_si_cce 

c     range for noise array in multiples of sigma for convolution with noise
      INTEGER*4 noise_range
      PARAMETER (noise_range=6)

      REAL*4 s_fano, s_fano_e, s_noise, s_noise_bin

c     define energy scale for unfolded spectrum
      INTEGER*4 e_max
      PARAMETER (e_max=32768)
      REAL*8 e_ax(0:e_max-1)
      REAL*8 spec_no_noise(0:e_max-1), spec_noise(0:e_max-1)
      REAL*4 e_min
      INTEGER*4 e_bin
c      REAL*4 e_bin
c     z- axis in nm
      INTEGER*4 zdim
      PARAMETER (zdim=1999)
      REAL*8 z_ax(0:zdim), cce_red(0:zdim), z_max
      INTEGER*4 l_ff, max_cce(0:zdim)

      REAL*8 min_e_1, max_e_1
      INTEGER*4 i_0, max_i_1, min_i_1
      REAL*4 rf_1
      INTEGER*4 f_0, f_1
      INTEGER*4 n_max
      PARAMETER (n_max=20000)

      REAL*8 kern, noise(0:n_max), noisearr(0:e_max-1)
      INTEGER*4 noise_length

      REAL*4 i_esc, a_l, e_k, e_esc
      REAL*4 i_fac, e_fac
      INTEGER*4 ceil
      REAL*4 rceil
      REAL*4 ehelp
      REAL*8 spec_no_noise_esc(0:e_max-1), spec_noise_esc(0:e_max-1)

      INTEGER*4 i, j, count
      LOGICAL     qdone
      REAL*4      y, dy, tmparrx(0:e_max-1), tmparry(0:e_max-1)
      REAL*8      yd, zd

C-HS  FFT variables
      COMPLEX*8   fftarr1(e_max), fftarr2(e_max)
      REAL*8      fftch(e_max*2), fftwa(e_max*2)
      INTEGER*4   fftifac(30)

c     binning of energy scale, in eV
      DATA e_bin/1/
c     Thickness of Si02 layer in nm
      DATA d_sio2/28./
c     Thickness of Si layer in nm
      DATA d_si/280000./
c     pair creation energy
c      DATA w/3.75/
c     new value used after May 6, 1999
      DATA w/3.70/
c     fano factor for noise
      DATA f_fano/0.12/
c     cte-noise estimation
      DATA s_cte/0./

c     parameters for escape - Peak
      DATA i_esc/0.03/
      DATA a_l/-4.307e-4/
      DATA e_k/1844./
      DATA e_esc/1740./

c     reset arrays
      do i=0,e_max-1
        spec_no_noise(i) = 0.d0
        spec_no_noise_esc(i) = 0.d0
        spec_noise(i) = 0.d0
        spec_noise_esc(i) = 0.d0
        tmparrx(i) = 0.
        tmparry(i) = 0.
        fftarr1(i+1) = (0.d0,0.d0)       ! C-RDS
        fftarr2(i+1) = (0.d0,0.d0)       ! C-RDS
      enddo
      do i=0,n_max
        noise(i) = 0.d0
        noisearr(i) = 0.d0
      enddo     
      do i=0,zdim
        z_ax(i) = 0.d0
        cce_red(i) = 0.d0
        max_cce(i) = 0
      enddo

c     convert input
      e_0 = dble(e_0_in)
      s_el = dble(s_el_in*w)
      l_ff = l_in
      c = dble(c_in)
      tau = dble(tau_in)
      s = dble(s_in)

c     fano-noise in eV
      s_fano =  sqrt(f_fano*w*e_0)

c     total noise the spectrum is folded with...
      s_noise = sqrt(s_fano**2+s_el**2+s_cte**2)

c     for fft convolution regard binning size!!
      s_noise_bin = s_noise/e_bin
c      write(*,*) f_fano, w, e_0, s_fano, s_el, s_cte, s_noise, e_bin

c     calculate fixed parameters for cce - function
      b_cce = (1.d0 - s)*(1.d0 - tau*c/(l_ff+tau*c))
      a_cce = (1.d0 - s)*tau*c/(l_ff+tau*c)

c     interpolate the actual absorption lengths...

C-HS  Using non - Numerical Recipes code
      call findindex(sienergy, nsiabs, e_0_in, i)
      if (i .eq. nsiabs) then
         a_si = dble(siabs(nsiabs))
      else
         call lininterp(sienergy(i), sienergy(i+1), siabs(i), 
     &        siabs(i+1), e_0_in, yvalue )
         a_si = dble(yvalue)
      endif

      call findindex(sio2energy, nsio2abs, e_0_in, i)
      if (i .eq. nsio2abs) then
         a_sio2 = dble(sio2abs(nsio2abs))
      else
         call lininterp(sio2energy(i), sio2energy(i+1), sio2abs(i),
     & sio2abs(i+1), e_0_in, yvalue )
         a_sio2 = dble(yvalue)
      endif      

c      write(*,*) e_0, a_si, a_sio2

c------------------------------------------------------------------------------+
c
c     Start of program
c
c------------------------------------------------------------------------------+

c     first, distribute number of photons to distinct layers

      n_ev = 100000.d0

c     assume rate of 100 ev/frame
      n_frames =n_ev/100.d0

c     number of pixels of one ccd
      num_pixels = 12800.d0

      if (n_ev.lt.n_frames*num_pixels) then
c       number of noise counts
        n_noise = n_frames*num_pixels - n_ev
      else
        n_noise = 0
C-HS    write(*,*) 'too many photons specified!'
      endif

c     number of photons which will not be recognized
      n_sio2_dead = 0.5*n_ev*(1.0-exp(-a_sio2*d_sio2))   !C-RDS

c     number of photons wich will give rise to a shelf count
      n_sio2_shelf = n_sio2_dead

      n_si_dead = 0.

c     number of photons which will be detected in the silicon,
c     to avoid floating underflow: exp - term cancelled (gives 0 anyway for E_0 < 3000 eV
      n_si = (n_ev - n_sio2_dead - n_sio2_shelf)*(1.-exp(-a_si*d_si))
c      write(*,*)n_si, n_sio2_dead, n_sio2_shelf

      do i = 0,l_ff
        z_ax(i) = float(i)
        cce_red(i) = s + b_cce*(z_ax(i)/l_ff)**c
      enddo
      do i = l_ff+1,zdim
        z_ax(i) = float(i)
        cce_red(i) = 1.d0 - a_cce*exp(-(z_ax(i)-l_ff)/tau)
      enddo

      count = 0
      y = (e_0 - float(e_bin))/e_0
c      write(*,*) y
c      write(*,*) cce_red
      do i = 0,zdim
        if (cce_red(i) .ge. y) then
          max_cce(count) = i        
          count = count+1
        endif
      enddo
      if (count .gt. 0) then
        z_max = z_ax(max_cce(0))
      else
        z_max = z_ax(zdim)
      endif

c     photons affected by cce (in measure of e_bin!)
c      write(*,*) z_max
      n_si_cce = n_si*(1.d0 - exp(-a_si*z_max))

c     photons not affected -> ideal response!
      n_rest = n_si-n_si_cce
c      write(*,*) n_si, n_si_cce

c      write(*,'(2f18.12)') (e_ax(i), cce_red(i), i=0,l_ff+5*tau)

c----------------------------------------------------------------------------------------------------
c     now, directly use formula for dn/de in two parts
c----------------------------------------------------------------------------------------------------

c     minimum energy in spectrum
      min_e_1 = e_0 * s

c     max energy for part one
      max_e_1 = e_0 * (s + b_cce)
c      write(*,*) min_e_1, max_e_1

c     for security reasons, check boundaries
      do i = 0,e_max-1
        if (e_ax(i).le.e_0) i_0 = i
      enddo
      do i = e_max-1,0,-1
        if (e_ax(i).ge.max_e_1) max_i_1 = i

c       MUST be gt in order to avoid running into singularity
        if (e_ax(i).gt.min_e_1) min_i_1 = i
      enddo
      max_i_1 = min(max_i_1-1,i_0-1)
      min_i_1 = min(min_i_1-1,max_i_1-1)
c      write(*,*) min_e_1, max_e_1
c      write(*,*) i_0,min_i_1, max_i_1
      if((max_i_1+1).gt.(e_max-1))then
        write(*,*)' increase e_max to at least ',max_i_1+2
        stop
      endif

      do i=min_i_1+1,max_i_1
        spec_no_noise(i) = e_bin*n_si*a_si*l_ff/(c*e_0*b_cce)* 
     &    exp(-a_si*l_ff*((e_ax(i)/e_0 - s)/b_cce)**(1.d0/c))*
     &    ((e_ax(i)/e_0 - s)/b_cce)**((1.d0/c)-1.d0)
      enddo
      do i=max_i_1+1,i_0-1
        spec_no_noise(i) = e_bin*n_si*exp(-a_si*l_ff)*a_si*tau/
     &    (e_0*a_cce**(tau*a_si))*(1.d0 - e_ax(i)/e_0)**(tau*a_si-1.d0)
      enddo
c      write(*,*) e_bin, n_si, a_si, tau
c      write(*,'(2f18.2)') (e_ax(i), spec_no_noise(i), i=0,e_max-1)

c----------------------------------------------------------------------------------------------------
c     add events in singularity and normalize this part of spectrum to n_si:
c----------------------------------------------------------------------------------------------------

      spec_no_noise(i_0) = n_rest
      yd = 0.d0
      do i = 0,e_max-1
        yd = yd + spec_no_noise(i)
      enddo
      do i = 0,e_max-1
        spec_no_noise(i) = spec_no_noise(i)*n_si/yd
      enddo
c      write(*,'(2f18.2)') (e_ax(i), spec_no_noise(i), i=0,i_0+5)

c----------------------------------------------------------------------------------------------------
c     add Sio2- Part:       use standard formula for linear f, take care of binning!!       
c----------------------------------------------------------------------------------------------------

      f_0 = 0
      rf_1 = min_e_1/(e_0*float(e_bin))
      l   = d_sio2

      do i=0,int(rf_1*e_0)-1
        spec_no_noise(i) = spec_no_noise(i) + 
     &    2.5d0*n_ev*a_sio2*l/(e_0*(rf_1-f_0))*         !C-RDS
     &    exp(-a_sio2*(e_ax(i)/e_0-f_0)*l/(rf_1-f_0))
      enddo
c      write(*,'(2f18.2)') (e_ax(i), spec_no_noise(i), i=0,i_0+5)

c----------------------------------------------------------------------------------------------------
c     Add Noise:
c----------------------------------------------------------------------------------------------------

c     convolution must be done with full noise, consisting of fano noise and electronic noise
c      write(*,'(a)') ' adding noise'

      noise_length = 2*int(noise_range*s_noise_bin)
c      write(*,*) noise_range, s_noise_bin, noise_length

      yd = 0.d0
      do i=0,noise_length-1
         kern = float(i) - float(noise_range)*s_noise_bin
         noise(i) = 1.d0/(sqrt(2.)*s_noise_bin)*
     &     exp(-kern**2/(2.d0*s_noise_bin**2))
         yd = yd + noise(i)
      enddo     

c      write(*,*) yd
c      write(*,*) noise

c     normalisation
      do i=0,noise_length-1
        noise(i) = noise(i) / yd
      enddo     
c      write(*,'(i8,2x,f18.14)') (i,noise(i), i=0,noise_length-1)

c     new method: FFT convolution:
c      write(*,'(a)') ' doing fft convolution...'
      do i=0,noise_length/2-1
        j = noise_length/2 + i
        noisearr(i) = noise(j)
        j = e_max/e_bin-noise_length/2 + i
        noisearr(j) = noise(i)
      enddo

c      write(*,'(i8,2x,f18.14)') (i,noisearr(i), i=0,e_max-1)

      do i=1,e_max
        fftarr1(i) = cmplx(spec_no_noise(i-1),0.d0)
        fftarr2(i) = cmplx(noisearr(i-1),0.d0)
      enddo

C-HS  Replaced numrec four1 calls with SLATEC cfftXX calls
      call cffti1(e_max, fftwa, fftifac)
      call cfftf1(e_max, fftarr1, fftch, fftwa, fftifac)
      call cfftf1(e_max, fftarr2, fftch, fftwa, fftifac)
      do i=1,e_max
        fftarr1(i) = fftarr1(i)*fftarr2(i)         
      enddo
      call cfftb1(e_max, fftarr1, fftch, fftwa, fftifac)

      do i=1,e_max
        spec_noise(i-1) = (fftarr1(i))*float(e_max/e_bin)
      enddo

      return
      end
