      subroutine pem_par(e_0_in, pattern, line, s_el_in, l_in, c_in, 
     &                   tau_in, s_in, cmode)

C
C subroutine to determine PEM parameters from polynomial fit results
C
C     Input energy and CCD line
C     Output PEM parameters: electronic noise, length of transition region, parameters c, tau, s
      REAL*4  e_0_in, line, s_el_in, l_in, c_in, tau_in, s_in
      integer*4  pattern
      character*2 cmode
      real*4  ds_dy, lblock, es

c     Parameters for FM from Orsay synchrotron (Temperature -110 deg C)
c     Parameters for FS from Orsay synchrotron (short QE, Temperature -90 deg C)
c                                  and fluorescence source (Temperature -90 deg C)
c     based now on official CTE model corrections 

      es = min(e_0_in,6000.)

c     noise increase with distance to CAMEX:
c     noise in number of electrons:
      if (cmode.eq.'ff') then
        if (pattern.eq.1) then
c	  2001 Oct. fitting lines of N132D
          ds_dy = 0.26 + 0.28*e_0_in*(4.9531E-5 - e_0_in*3.6396E-10)
        elseif (pattern.eq.2) then
          ds_dy = 0.26 + 5.04*e_0_in*(4.9531E-5 - e_0_in*6.5513E-10)
        endif
        lblock = (line-9.5)/20.0
	s_el_in = 6.05 + lblock*ds_dy
      elseif (cmode.eq.'ef') then
        if (pattern.eq.1) then
c         2001 july 23 (parameters derived from cal source, extended!)
          ds_dy = 0.26 + 0.28*e_0_in*(4.9531E-5 - e_0_in*3.6396E-10)
        elseif (pattern.eq.2) then
          ds_dy = 0.26 + 5.04*e_0_in*(4.9531E-5 - e_0_in*6.5513E-10)
        endif
        lblock = (line-9.5)/20.0
c       s_el_in = 6.863 + lblock*ds_dy
c       2001 Aug 22 (adjustment to fit OVII-line complex in Vela)
        s_el_in = 5.5 + lblock*ds_dy
      elseif (cmode.eq.'sw') then
        if (pattern.eq.1) then
c	  2001 june 16
          ds_dy = 0.26 + 0.28*e_0_in*(4.9531E-5 - e_0_in*3.6396E-10)
       elseif (pattern.eq.2) then
          ds_dy = 0.26 + 5.04*e_0_in*(4.9531E-5 - e_0_in*6.5513E-10)
        endif
        lblock = (line-44.5)/20.0
	s_el_in = 6.8 + lblock*ds_dy
      elseif (cmode.eq.'lw') then
        if (pattern.eq.1) then
c	  2001 june 16
          ds_dy = 1.00 + 0.28*e_0_in*(4.9531E-5 - e_0_in*3.6396E-10)
        elseif (pattern.eq.2) then
          ds_dy = 0.26 + 5.04*e_0_in*(4.9531E-5 - e_0_in*6.5513E-10)
        endif
        lblock = (line-19.5)/20.0
        s_el_in = 6.863 + lblock*ds_dy
      elseif (cmode.eq.'ti') then
        if (pattern.eq.1) then
          ds_dy = 0.26 + 0.28*e_0_in*(4.9531E-5 - e_0_in*3.6396E-10)
        elseif (pattern.eq.2) then
          ds_dy = 0.26 + 5.04*e_0_in*(4.9531E-5 - e_0_in*6.5513E-10)
        endif
        lblock = (line-9.5)/20.0
        s_el_in = 5.783 + lblock*ds_dy
      endif

      
c     PEM parameter c for FM and FS:
      c_in = 1.0

c     PEM parameter tau for FM:
c     tau_in = 110.2 - es*1.6275E-2

c     PEM parameter tau for FS:
c version 4.0, 20.11.00 to fit RXJ0720 (wrong due to pile up):
c     tau_in = 145. - es*1.7E-2
c back to previous value, adjust later when CTE is finalized
c     with this tau the nH for MS1229.2 goes below galactic when fitting above 0.7 keV
c     tau_in = 110.2 - es*1.6275E-2
c 12.06.2001 fit to values measured on spare detector at PANTER
      tau_in = 92.16 - es*1.0664E-2

c     PEM parameter s for FM:
c      s_in = 0.6619 - 0.4342*exp(-(e_0_in+481.7)/2070.0)

c     PEM parameter s for FS:
c     s_in = 0.9237 - 0.7904*exp(-(e_0_in+151.19)/1746.8)
c     s_in = 0.6619 - 0.4342*exp(-(e_0_in+481.7)/2070.0)
c 12.06.2001 fit to values measured on spare detector at PANTER
      s_in = 0.6173 - 0.4597*exp(-(e_0_in+424.9)/1107.0)

c     PEM parameter l for FS:
      if (e_0_in.lt.1844.) then
        l_in = 55.59 + 68.90*exp(-(e_0_in-77.23)/537.4)
      else
        l_in = 81.37 + 224.0*exp(-(e_0_in-987.2)/375.2)
      endif

c      if (cmode.eq.'sw') then
c       test to improve the residuals around 0.5 keV in Mkn421
c        tau_in = (110.2 - es*1.6275E-2)*1.15
c      endif
       
c      write(*,'(f7.1,4f7.2,f7.4)') 
c     &        e_0_in, s_el_in, l_in, c_in, tau_in, s_in
      return
      end
